//--------------------------------------//
// Bleeder                              //
// Players bleed where they take damage //
//--------------------------------------//
class B expands Mutator;

var bool Initialized;
var Pawn PlayerList[32];
var BleederEffectSpawner PlayerBleederList[32], besBufferList[32];

// Re-setup pawn id's if a new player enters, courtesy of [KiX] ProfessorGib
function ModifyPlayer(Pawn other)
{       
	ListPlayers();
	if (NextMutator != None) NextMutator.ModifyPlayer(other);
}

// Assigns each pawn a unique id, with help from examples by [KiX] ProfessorGib 
function ListPlayers()
{
	local Pawn P;
	local int i, j;

	// save players to list
	i = 0;
	for (P = Level.PawnList; P != None; P = P.NextPawn)
	{
		PlayerList[i] = P;
		i++;
	}

	// reassign index entries of players' bleeder and numbleeder lists
	// first wipe buffer lists
	for (i = 0; i < 32; i++) besBufferList[i] = None;
	// now scan current lists and resort into proper order based on pawn list
	for(i = 0; i < 32; i++)
	{
		// make sure we don't overstep the valid bounds within the playerlist
		if (PlayerList[i] == None) break;
		// give a valid pawn, find the bleeders which go to it (if any) and save them to the appropriate buffer list
		for (j = 0; j < 32; j++)
		{
			// check if player isn't bleeding on us
			if (PlayerBleederList[j] == None) break;
			// we have a match
			if (PlayerBleederList[j].Owner == PlayerList[i])
			{
				besBufferList[i] = PlayerBleederList[j];
			}
		}
		// now to wipe the actual lists and copy buffers to them
		for (i = 0; i < 32; i++)
		{
			PlayerBleederList[i] = besBufferList[i];
		}
	}
}

function PostBeginPlay()
{
	local int i;

	if (Initialized) return;
	Initialized = True;

	// init all the BleederEffectSpawner's
	for (i = 0; i < 32; i++)
	{
		PlayerBleederList[i] = None;
	}
	
	Level.Game.RegisterDamageMutator(Self);

	Super.PostBeginPlay();
}

function ScoreKill(Pawn Killer, Pawn Other)
{
	local int i, j;
	local BleederEffectSpawner BES, tBES;

	// find the dead man in our list
	for (i = 0; i < 32; i++)
	{
		if (PlayerList[i] == Other) break;
	}
	// paranoia check that we found the player (no reason we shouldn't)
	if ((i != 32) && (PlayerList[i] != None))
	{
		// check if there are any BleederEffectSpawns attached to this player
		if (PlayerBleederList[i] != None)
		{
			// delete and destroy all bleeder spawners for this player
			tBES = PlayerBleederList[i];
			PlayerBleederList[i] = None;
			while (tBES != None)
			{
				BES = tBES.NextSpawner;
				// no need to be kind and rewind - just kill it outright
				tBES.Destroy();
				tBES = BES;
			}
		}
	}

	Super.ScoreKill(Killer, Other);
}

function MutatorTakeDamage(out int ActualDamage, Pawn Victim, Pawn InstigatedBy, out Vector HitLocation, out Vector Momentum, name DamageType)
{
	local BleederEffectSpawner BES;
	local int i, j, D;

	if (ActualDamage > 0)
	{
		for (i = 0; i < 32; i++)
		{
			if (PlayerList[i] == Victim)
			{
				BES = Spawn(class'BleederEffectSpawner', Victim, , HitLocation, Victim.Rotation);
				if (BES != None)
				{
					BES.InitSpawner();
					if (ActualDamage > 30) j = 0;
					else if (ActualDamage > 20) j = 10;
					else j = Rand(10) + 1;
					BES.SetSpawnProperties(0.5, j, j, class'BloodSpray', true);
					if (PlayerBleederList[i] == None) PlayerBleederList[i] = BES;
					else PlayerBleederList[i] = BES.AddToList(PlayerBleederList[i]);
					BES = None;
				}
				break;
			}
		}
	}

	Super.MutatorTakeDamage(ActualDamage, Victim, InstigatedBy, HitLocation, Momentum, DamageType);
}

defaultproperties
{
}
