//-------------------------------------------------------------------//
// BleederEffectSpawner                                              //
// this class handles the spawning of the bleeding effect on players //
//-------------------------------------------------------------------//

class BleederEffectSpawner extends Actor;

var float SpawnDelay;		// time between effect spawns
var float DelayLeft;		// time left until next spawn
var int NumSpawns;		// number of total effects to spawn. 0 means infinite
var int SpawnsLeft;		// number of effects left to spawn before the spawner is destroyed
var bool bIsSpawning;		// flag to check whether spawner is active
var vector RelLocation;		// the location of the spawner relative to the location of the owner
var BleederEffectSpawner FirstSpawner;	// first spawner in the list
var BleederEffectSpawner NextSpawner;	// next spawner in the list
var class<Effects> EffectToSpawn;	// the effect we want to spawn

function bool IsSpawning()
{
	return bIsSpawning;
}

function StopSpawner()
{
	SetTimer(0.0, false);
	bIsSpawning = false;
}

function StartSpawner()
{
	// check if we're already spawning
	if (bIsSpawning)
	{
		// stop spawning and reset vars where pertinent
		StopSpawner();
		SpawnsLeft = NumSpawns;
	}
	bIsSpawning = true;
	SetTimer(SpawnDelay, true);
}

function SetRelativeLocation()
{
	RelLocation = Location - Owner.Location;
}

function InitSpawner()
{
	SetRelativeLocation();
	FirstSpawner = Self;
	NextSpawner = None;
}

function SetSpawnProperties(optional float SD, optional int NS, optional int SL, optional class<Effects> E, optional bool RestartTimer)
{
	SpawnDelay = SD;
	NumSpawns = NS;
	SpawnsLeft = SL;
	EffectToSpawn = E;
	if (RestartTimer) StartSpawner();

	return;
}

function Timer()
{
	// only process if we're spawning
	if (bIsSpawning)
	{
		// check if we're spawning a finite number
		if (NumSpawns > 0)
		{
			SpawnsLeft--;
			Spawn(EffectToSpawn, Self, , RelLocation + Owner.Location, Owner.Rotation);
//			Log("Bleeder bled from "$Owner.Name);
			// we've run out of spawns
			if (SpawnsLeft <= 0)
			{
				DeleteFromCurrentList();
				Destroy();
			}
		}
		else Spawn(EffectToSpawn, Owner, , RelLocation + Owner.Location, Owner.Rotation);
	}
}

function BleederEffectSpawner AddToList(BleederEffectSpawner FS)
{
	local BleederEffectSpawner BES;

	// make sure this is valid
	if (FS == None) return None;

	// we are an unlimited spawner
	if (NumSpawns == 0)
	{
		// the list already has unlimited spawners
		if (FS.NumSpawns == 0)
		{
			FirstSpawner = FS;
			NextSpawner = FS.NextSpawner;
			FS.NextSpawner = Self;
		}
		// need make us the first spawner
		else
		{
			NextSpawner = FS;
			BES = Self;
			while (BES != None)
			{
				BES.FirstSpawner = Self;
				BES = BES.NextSpawner;
			}
		}
	}
	// we are a limited spawner
	else
	{
		// we go at the end of the list
		if (FS.NumSpawns == 0)
		{
			FirstSpawner = FS;
			BES = FS;
			while (BES.NextSpawner != None) BES = BES.NextSpawner;
			NextSpawner = BES.NextSpawner;
			BES.NextSpawner = NextSpawner;
		}
		// throw us right next to the first spawner
		else
		{
			FirstSpawner = FS;
			NextSpawner = FS.NextSpawner;
			FS.NextSpawner = Self;
		}
	}

	return FirstSpawner;
}

function BleederEffectSpawner DeleteFromCurrentList()
{
	local BleederEffectSpawner BES, tBES;

	// check for first entry condition
	if (FirstSpawner == Self)
	{
		// set the other entries first node to NextSpawner
		BES = NextSpawner;
		tBES = NextSpawner;
		while (BES != None)
		{
			BES.FirstSpawner = tBES;
			BES = BES.NextSpawner;
		}
		NextSpawner = None;
	}
	// find the list node before this and connect it to the one after this, eliminating this entry from the list
	else
		if (BES != None)
	{
		BES = FirstSpawner;
		tBES = FirstSpawner;
		while (BES.NextSpawner != None)
		{
			if (BES.NextSpawner == Self) break;
			BES = BES.NextSpawner;
		}
		if (BES.NextSpawner != None)
		{
			BES.NextSpawner = NextSpawner;
		}
		BES = None;
		NextSpawner = None;
		FirstSpawner = Self;
	}

	return tBES;
}

defaultproperties
{
     bHidden=True
}
